* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*  (C) 1992-97 LEIF E. PETERSON. ALL RIGHTS RESERVED.               *
*  PC FORTRAN COMPUTER PROGRAM TO ESTIMATE REGRESSION COEFFICIENTS, *
*  STANDARD ERRORS, PEARSON CHI-SQUARE RESIDUALS, DEVIANCE, FREEMAN *
*  -TUKEY RESIDUALS AND LEVERAGES FOR A LINEAR, LOG-LINEAR, OR NON- *
*  LINEAR SET OF POISSON DISTRIBUTED DATA.                          *
*  DESIGN AND CODE:  LEIF E. PETERSON, PH.D.                        *
*                    CENTER FOR CANCER CONTROL RESEARCH             *
*                    BAYLOR COLLEGE OF MEDICINE                     *
*                    ONE BAYLOR PLAZA, ST-924                       *
*                    HOUSTON, TEXAS 77030                           *
*  REV: 7/27/97                                                     *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C
C  SUBROUTINES:
C  INIT    - PERFORMS STARTING REGRESSION TO ESTIMATE INITIAL COEFFICIENTS
C  PRED    - PREDICTS YHAT,DEV.,CHI-SQ.,FREEMAN-TUKEY,AND LEVERAGES.
C  MATMUL  - MULTIPLIES INPUT MATRIX
C  TRNPOS  - TRANSPOSES INPUT MATRIX 
C  SVDCMP  - SINGULAR VALUE DECOMPOSITION (USED TO INVERT X'WX MATRIX)
C  ALG     - CHI-SQUARE GOODNESS OF FIT
C  GAMAIN  - CHI-SQUARE GOODNESS OF FIT
C  FIT     - ESTIMATES LEVERAGES, DELETION RESIDUALS, ADJUSTED RESIDUALS 
C
C  ARRAYS:
C  START(I)    - STARTING VALUES FOR PARAMETERS (NON-LINEAR MODEL) 
C  OBS(I)      - NUMBER OF DEATHS IN SUBGROUP I
C  LAMBDA(I)   - OBSERVED DEATH RATE IN GROUP I (DEATHS/PERSON YEARS)
C  PY(I)       - PERSON YEARS IN GROUP I
C  X(J)        - ROW VECTOR OF DESIGN MATRIX, X
C  XT(J,I)     - TRANSPOSE OF X MATRIX FOR FIT SUBROUTINE
C  LAMHAT(I)   - PREDICTED RATE (LAMDA HAT)
C  EXPE(I)     - EXPECTED NUMBER OF DEATHS IN SUBGROUP I 
C  Y(I)        - DIFFERENCE OF OBSERVED AND PREDICITED NUMBER OF DEATHS
C  W(I)        - WEIGHTS
C  Z(I,J)      - PARTIAL DERIVATIVES (FOR NON-LINEAR REGRESSION)
C  ZTWZ(J,J)   - INFORMATION MATRIX
C  ZTWZ1(J,J)  - INVERSE OF INFORMATION MATRIX
C  ZTWY(J,1)   - LIKELIHOOD EQUATIONS
C  FT(I)       - COLUMN VECTOR OF FREEMAN-TUKEY RESIDUALS
C  PR(I)       - COLUMN VECTOR OF PEARSON CHI-SQUARE RESIDUALS
C  VARCOV(J,J) - VARIANCE-COVARIANCE MATRIX (INVERSE OF ZTWZ MATRIX)
C  G(J,1)      - COLUMN VECTOR OF SOLUTIONS TO LINEAR EQUATIONS
C  B(J,1)      - REGRESSION COEFFICIENTS (UPDATED AT EACH ITERATION WITH G)
C  SE(J)       - COLUMN VECTOR OF STANDARD ERRORS OF REGRESSION COEFFICIENTS
C  WALD(J)     - WALD STATISTICS (BETA/S.E.(BETA))
C  CORR(J,J)   - CORRELATION MATRIX OF COEFFICIENTS
C  LL(J)       - LOWER LIMIT OF 95% CONF. INTERVAL FOR MULTIPLICATIVE MODEL.
C  UL(J)       - UPPER LIMIT OF 95% CONF. INTERVAL FOR MULTIPLICATIVE MODEL.
C  ML(J)       - POINT ESTIMATE OR RELATIVE RISK FOR MULTIPLICATIVE MODEL.
C  LEV(J,J)    - LEVERAGES
C  DRES(J,J)   - DELETION RESIDUALS
C  SRES(J)     - ADJUSTED RESIDUALS
C  
C  VARIABLES:
C  NROW        - NUMBER OF INPUT RECORDS
C  NCOL        - NUMBER OF PARAMETERS TO BE ESTIMATED
C  NREG        - LINEAR, LOG-LINEAR, OR NON-LINEAR POISSON REGRESSION
C  RHO         - POWER TRANSFORM FOR POWER FAMILY OF MODELS
C  OUTFIT      - MODEL TO BE FIT
C  
      PARAMETER (MP=500,NP=28,IP=1,NIP=16)
      DIMENSION VARCOV(NP,NP),ZTWZ1(NP,NP),ZTWY(NP,IP),
     1 G(NP),B(NP),CORR(NP,NP),BIN(NP),SE(NP),WALD(NP)
      REAL LL(NP),ML(NP),UL(NP),LEV(MP,MP),XIN(MP,NP),WIN(MP,MP),
     1 YDIFF(MP),XT(NP,MP),OEXPE(MP),SRES(MP),DRES(MP,NP),
     2 OOBS(MP),OFT(MP),OPR(MP),START(NP)
      CHARACTER GP(30)*3,DLINE*4,INFILE*50,OUTFILE*50
      DATA DLINE/'(//)'/
      DATA GP/'B1','B2','B3','B4','B5','B6','B7','B8','B9',
     1 'B10','B11','B12','B13','B14','B15','B16','B17','B18','B19',
     1 'B20','B21','B22','B23','B24','B25','B26','B27','B28','B29',
     1 'B30'/
      CALL QUERY(NREG,RHO,IPRED,NCOL,ISTART,START,IEX,
     1 ICOVAR,IDATA,ICELLS,IREG,INFILE,OUTFILE,IFLT)
      IF(IFLT.NE.0)CALL FAULT(IFLT)
      OPEN(2,FILE=INFILE,STATUS='OLD')
      OPEN(3,FILE=OUTFILE,STATUS='UNKNOWN')
480   CONTINUE
      IF(NREG.EQ.3)THEN
      ITER=0
      ITERP=ITERP+1
      RHO=1.1000-(FLOAT(ITERP)*.1)
      IF(ITERP.EQ.10)RHO=0. 
      ENDIF
      CALL INIT(NREG,NCOL,NROW,ISTART,START,B,G,
     1 RHO,IPRED,IDATA,IFLT)
      IF(IFLT.NE.0)CALL FAULT(IFLT)
501   CONTINUE
      ITER=ITER+1
      IF(ITER.GT.100)CALL FAULT(6)
C
C  ESTIMATE THE RESIDUALS (OBSERVED Y-VALUES MINUS THE PREDICTED OR
C  EXPECTED) AND CALCULATE THE COLUMN VECTOR OF YHATS.  DETERMINE THE 
C  W MATRIX WITH DIAGONAL ELEMENTS REPRESENTING THE RATIO OF PERSON 
C  YEARS TO YHAT (EXPECTED RATE).
C
      CALL PRED(NCOL,NROW,G,IDF,DEV,TPR,NREG,RHO,IPRED,
     1 ZTWZ1,ZTWY,WIN,YDIFF,OEXPE,OOBS,OFT,OPR,XIN,XT,IFLT)
      IF(IFLT.NE.0)CALL FAULT(IFLT)
C
C    OBTAIN CHI-SQUARE PROBABILITIES FOR CHI-SQUARE AND DEVIANCE
C
C    DF - DEGREES OF FREEDOM
C    CHI2 - VALUE OF CHI-SQUARE
      IF(TPR.LT.4*NROW)THEN
      AX = ALG(FLOAT(IDF)*.5,IER)
      CSPROB = 1.-GAMAIN(TPR*.5,FLOAT(IDF)*.5,AX,IER)
      DVPROB = 1.-GAMAIN(DEV*.5,FLOAT(IDF)*.5,AX,IER)    
      ENDIF
      DO 25 IL=1,NCOL
      DO 20 IJ=1,NCOL
      VARCOV(IJ,IL)=ZTWZ1(IJ,IL)
20    CONTINUE
      IF(VARCOV(IL,IL).GT.0.)SE(IL)=SQRT(VARCOV(IL,IL))
25    CONTINUE
      L=NCOL+1
      IPTER=ITER-1
      IFLAG=0
      IF(ITER.EQ.1)WRITE(3,DLINE)
      WRITE(*,*)' '
      WRITE(6,30)ITER,DEV
      WRITE(3,30)ITER,DEV
30    FORMAT(T5'ITERATION: ',I3,6X'DEVIANCE: ',F17.4)
      WRITE(*,*)' '
C
C REVISE PARAMETER ESTIMATES BY ADDING THE BETA COEFFICIENTS TO
C THE G-COEFFICIENTS (MODEL PARAMETERS) AND CHECK FOR CONVERGENCE 
C ON ALL PARAMETERS WITH A CRITERION OF < 1.E-5.
C
      IFLAG=0
      DO 90 J=1,NCOL
      IF(ABS(G(J)).LE.(1.E-5*ABS(B(J)+1.E-7)))IFLAG=IFLAG+1      
      B(J)=B(J)+G(J)
      BIN(J)=B(J)
90    CONTINUE
      IF(IFLAG.EQ.NCOL)GO TO 502
      DO 91 J=1,NCOL
91    G(J)=B(J)
      GO TO 501
502   CONTINUE
      IF(IREG.EQ.1)CALL FIT(WIN,YDIFF,XIN,XT,ZTWZ1,OEXPE,LEV,DRES,
     1  SRES,NCOL,NROW)
      IF(NREG.NE.3.AND.ICELLS.EQ.1.)THEN
      WRITE(3,102)ITER
102   FORMAT(/T5'ITERATIONS: ',I3,/T5'REC',T19'OBS',13X'EXP',
     1 8X,'FREEMAN-TUKEY',6X,'PEARSON'/)
      DO 103 J=1,NROW
      WRITE(3,104)J,OOBS(J),OEXPE(J),OFT(J),OPR(J)
104   FORMAT(2X,I4,2F16.1,3F16.4)
103   CONTINUE
      ENDIF
C
C WRITE COEFFICIENTS AND THEIR STANDARD ERRORS
C
      WRITE(3,105)
105   FORMAT(//T11'B',6X,'COEFFICIENT   STD. ERROR        WALD',
     1'          SCORE',/)
      DO 107 I=1,NCOL
      IF(SE(I).GT.0.)WALD(I)=B(I)/SE(I)
      WRITE(3,106)I,B(I),SE(I),WALD(I),ZTWY(I,1)
106   FORMAT(T10,I2,8X,F8.4,4X,F8.4,4X,F12.4,3X,F12.8)
107   CONTINUE
C
C FOR LOG-LINEAR REGRESSION, IF USER SPECIFIES EXPONENTIATION
C OF COEFFICIENTS TO OBTAIN RELATIVE RISK, THEN EXPONENIATE AND
C ESTIAMTE THEIR 95% CONFIDENCE INTERVALS BY ADDING TO OR SUBTRACTING
C FROM THE COEFFICIENT ITS STANDARD ERROR (s.e.) ADJUSTED FOR A
C TYPE I ERROR (ALPHA=0.05,1.96).
C     
      IF(IEX.EQ.1)THEN
      WRITE(3,118)
118   FORMAT(//T11'B',6X,'COEFFICIENT   STD. ERROR   RELATIVE',
     1 ' RISK',8X,' 95% CONF. INT. ',/)
      DO 119 I=1,NCOL
      LL(I)=EXP(B(I)-(1.96*SE(I)))
      UL(I)=EXP(B(I)+(1.96*SE(I)))
      ML(I)=EXP(B(I))
      WRITE(3,120)I,B(I),SE(I),ML(I),LL(I),UL(I)
120   FORMAT(T10,I2,8X,F8.4,4X,F8.4,9X,F8.2,8X,'(',F8.2,',',F8.2,')')
119   CONTINUE
      ENDIF
      WRITE(3,128)RHO
128   FORMAT(//T9'POWER: ',F6.2,//T9'GOODNESS-OF-FIT TESTS'//,
     1 T9'STATISTIC',T21'ESTIMATE',T33'D.F.',T46'PROB.') 
      WRITE(3,129)TPR,IDF,CSPROB,DEV,IDF,DVPROB
129   FORMAT(/T9'CHI-SQUARE'T19,F9.3,T32,I5,T42,F9.4,/,
     1 T9'DEVIANCE',T19,F9.3,T32,I5,T42,F9.4)
      IF(ICOVAR.EQ.1.)THEN
      WRITE(3,130)(GP(KN),KN=1,NCOL)
130   FORMAT(//T5'VARIANCE-COVARIANCE MATRIX OF COEFFICIENTS.',//,
     1T10,30(A3,7X))
      DO 131 J=1,NCOL
      WRITE(3,132)GP(J),(VARCOV(J,IN),IN=1,J)
132   FORMAT(T2,A3,2X,30(F7.3,3X))
131   CONTINUE
      ENDIF
C
C WRITE THE CORRELATION MATRIX FOR THE COEFFICIENTS.  THE CORRELATION 
C BETWEEN TWO VARIABLES IS DETERMINED BY DIVIDING THE COVARIANCE BETWEEN 
C THE TWO VARIABLES BY THE PRODUCT OF THE THEIR STANDARD ERRORS.
C
      IF(ICOVAR.EQ.1)THEN
      WRITE(3,113)(GP(KN),KN=1,NCOL)
113   FORMAT(//T5'CORRELATION MATRIX OF COEFFICIENTS.',//,
     1T10,11(A2,8X))
      DO 115 J=1,NCOL
      DO 114 K=1,NCOL
114   CORR(J,K)=VARCOV(J,K)/(SE(J)*SE(K))
      WRITE(3,116)GP(J),(CORR(J,IN),IN=1,J)
116   FORMAT(T2,A2,2X,11(F7.3,3X))
115   CONTINUE
      ENDIF
      IF(IREG.EQ.1)THEN
      WRITE(3,222)
222   FORMAT(///T8'TRACE OF THE HAT MATRIX AND ADJUSTED RESIDUALS.',
     1 /,T8'RECORD         LEVERAGE      ADJ. RESID.',/)
      DO 223 J=1,NROW
      WRITE(3,224)J,LEV(J,J),SRES(J)
224   FORMAT(8X,I5,2X,2F15.4)
223   CONTINUE
      WRITE(3,225)(GP(J),J=1,NCOL)
225   FORMAT(///T5'DELETION RESIDUALS OR APPROXIMATE CHANGE IN',
     1 ' COEFFICIENTS AFTER',/T5'DELETION OF EACH RECORD.',//,
     2 T5'RECORD',8X,11(A3,5X),/)
      DO 228 J=1,NROW
      WRITE(3,227)J,(DRES(J,K),K=1,NCOL)
227   FORMAT(T6,I5,4X,10(F7.4,1X))
228   CONTINUE
      ENDIF
      IF(NREG.EQ.3.AND.RHO.GT.0.)GO TO 480
      WRITE(6,600)OUTFILE
600   FORMAT(//T2'RUN WAS SUCCESSFUL - OUTPUT IN FILE: ', A50)
      CLOSE(2)
      CLOSE(3)
      STOP
      END

      SUBROUTINE QUERY(NREG,RHO,IPRED,NCOL,ISTART,START,IEX,
     1 ICOVAR,IDATA,ICELLS,IREG,INFILE,OUTFILE,IFLT)
      PARAMETER (NP=28)
      CHARACTER*50 INFILE,OUTFILE
      DIMENSION START(NP)
C
C  PROMPT USER FOR THE NUMBER OF PAREMETERS TO BE FITTED AND 
C  THEIR INITIAL VALUES.
C
      ISTART=1
      WRITE(*,*)'ENTER THE NAME OF INPUT FILE: '
      READ(*,'(A50)')INFILE
      WRITE(*,*)'ENTER THE NAME OF OUTPUT FILE: '
      READ(*,'(A50)')OUTFILE
      WRITE(*,*)' SELECT MODEL:  1-ADD 2-MULT 3-POWER 4-NON-LINEAR'
      READ(*,*)NREG
      IF(NREG.EQ.1)THEN
      WRITE(*,*)' ENTER RHO:  1 - ADDITIVE     - OTHER'
      READ(*,*)RHO
      ENDIF
      WRITE(*,*)' ENTER NUMBER OF PREDICTOR (INPUT) VARIABLES'
      READ(*,*)IPRED
      WRITE(*,*)' NUMBER OF PARAMETERS TO BE FITTED?'
      READ(5,'(I2)')NCOL
      IF(NREG.EQ.4)THEN
      WRITE(*,*)' INPUT OR ESTIMATE STARTING VALUES: 0-INPUT 1-EST'
      READ(5,'(I2)')ISTART
      IF(ISTART.EQ.0)THEN
      DO 5 I=1,NCOL
      WRITE(*,*)' INPUT STARTING VALUE FOR COEFFICIENT',I
      READ(5,'(F12.7)')START(I)
5     CONTINUE
      ENDIF
      ENDIF
      IF(NREG.EQ.2)THEN
      WRITE(*,*)'EXPONENTIATE COEFFICIENTS?  0-NO   1-YES'
      READ(*,*)IEX
      ENDIF
      WRITE(*,*)' PRINT INFORMATION AND CORR MATRICES?  0-NO   1-YES'
      READ(*,*)ICOVAR
      WRITE(*,*)' PRINT INPUT DATA?  0-NO   1-YES'
      READ(*,*)IDATA
      WRITE(*,*)' PRINT FITTED CELLS?  0-NO   1-YES'
      READ(*,*)ICELLS
      WRITE(*,*)' PERFORM REGRESSION DIAGNOSTICS?  0-NO   1-YES'
      READ(*,*)IREG
      IF(NCOL.LT.1)IFLT=1
      RETURN
      END
      
      SUBROUTINE INIT(NREG,NCOL,NROW,ISTART,START,B,G,RHO,IPRED,
     1 IDATA,IFLT)
      PARAMETER(MP=500,NP=28,IP=1)
      DIMENSION X(NP),ZTWZ(NP,NP),ZTWZ1(NP,NP),ZTWY(NP,IP),G(NP),
     1 B(NP),START(NP)
      REAL*4 LAMBDA,BUFFER
      CHARACTER IFMT*55,OFMT*55,ADD*4,OUTFIT(4)*14,TITLE*30
      DATA ADD/'(T6,'/
      DATA OUTFIT/'ADDITIVE','MULTIPLICATIVE','POWER','NON-LINEAR'/
      REWIND 2
C
C ZERO ARRAYS.
C
      DO 1 L=1,NCOL
      DO 2 M=1,NCOL
2     ZTWZ(L,M)=0.
      ZTWY(L,1)=0.
1     CONTINUE
      IF(NREG.EQ.4.AND.ISTART.EQ.0.AND.IDATA.EQ.0)GO TO 30
      READ(2,3)TITLE,IFMT
3     FORMAT(A30,/,A50)
      OFMT(4:55)=IFMT
      OFMT(1:4)=ADD
      WRITE(3,4)TITLE,OUTFIT(NREG),IFMT
4     FORMAT(//T5'TITLE: ',A30,
     1 //T5'MODEL SELECTED: 'A14,//,T5'INPUT DATA FORMAT: ',A55,//)
5     CONTINUE
      I=I+1
      WRITE(6,1000)I
1000  FORMAT('+','Records read: ',I6)
      READ(2,IFMT,END=500)OBS,PY,(X(J),J=1,IPRED)
      IF(IDATA.EQ.1.)WRITE(3,OFMT)OBS,PY,(X(J),J=1,IPRED)
C
C FOR NON-LINEAR MODELS (NREG=4), IF USER SPECIFIES STARTING
C VALUES FOR PARAMETERS, THEN SKIP THE DEFAULT MULTIPLICATIVE FIT,
C AND CONTINUE READING DATA IN CASE THE USER SPECIFIED THAT INPUT DATA
C ARE TO BE WRITTEN TO OUTPUT FILE (IDATA=1).  tHIS IS ACCOMPLISHED BY
C ADVANCING TO LINE 30. 
C
      IF(NREG.EQ.4.AND.ISTART.EQ.0.AND.IDATA.EQ.1)GO TO 5
      IF(PY.GT.0.)LAMBDA=OBS*(1./PY)
      IF(NREG.EQ.2.OR.NREG.EQ.4)RHO=0.
C
C  ITERATE THE LEAST SQUARES FIT, EACH TIME ADDING THE REGRESSION 
C  COEFFICIENTS TO THE B ESTIMATES.
C
      IFAULT=0
      IF(RHO.NE.0.)Y=LAMBDA**RHO
      IF(RHO.EQ.0..AND.LAMBDA.GT.0.)Y=ALOG(LAMBDA)
      W=PY
C
C CALCULATE DISPERSION MATRICES.
C
      DO 20 L=1,NCOL
      DO 15 M=1,NCOL
      ZTWZ(L,M)=ZTWZ(L,M)+X(L)*W*X(M)
15    CONTINUE
20    CONTINUE
      DO 22 L=1,NCOL
      ZTWY(L,1)=ZTWY(L,1)+X(L)*W*Y
22    CONTINUE
C
C READ NEXT INPUT RECORD, WHEN ALL RECORDS ARE READ GO TO LINE 500.  
C
      GO TO 5
30    CONTINUE
500   NROW=I-1
C
C IF STARTING VALUES FOR NON-LINEAR MODEL WERE INPUT BY USER IN "QUERY"
C SUBROUTINE, WRITE OUTPUT FILE HEADER AND SKIP PURELY MULTIPLICATIVE
C FIT TO ESTIMATE INITIAL PARAMETERS, AND SET G(.) AND B(.) EQUAL TO
C START(.).  THEN RETURN TO MAIN PROGRAM.
C
      IF(NREG.EQ.4.AND.ISTART.EQ.0)THEN
      WRITE(3,6)
6     FORMAT(/T5'STARTING VALUES FOR PARAMETERS',//,
     3 T5'PARAMETER   STARTING VALUE'/)
      DO 8 I=1,NCOL
      WRITE(3,7)I,START(I)
7     FORMAT(7X,I2,7X,F12.7) 
      G(I)=START(I)
      B(I)=G(I)
8     CONTINUE
      RETURN
      ENDIF
C
C INVERT WEIGHTED DISPERSIAN MATRIX.
C
      CALL MATINV(ZTWZ,ZTWZ1,NCOL)
      CALL MATMUL(ZTWZ1,ZTWY,G,NCOL,NCOL,IP,MP,NP,IP)
      DO 25 K=1,NCOL
      B(K)=G(K)
      BUFFER=BUFFER+ABS(B(K))
25    CONTINUE
      IF(NROW.LT.1)IFLT=2
      IF(NROW-NCOL.LT.1)IFLT=3
      IF(BUFFER.EQ.0.)IFLT=4
      RETURN
      END

      SUBROUTINE PRED(NCOL,NROW,G,IDF,DEV,TPR,NREG,
     1 RHO,IPRED,ZTWZ1,ZTWY,WIN,YDIFF,OEXPE,OOBS,OFT,OPR,
     1 XIN,XT,IFLT)
      PARAMETER (MP=500,NP=28,IP=1)
      DIMENSION X(NP),Z(NP),ZTWZ(NP,NP),ZTWZ1(NP,NP),ZTWY(NP,IP),
     1 G(NP),PROD1(NP),PROD2(NP),XT(NP,MP),
     2 XIN(MP,NP),YDIFF(MP),OEXPE(MP),WIN(MP,MP),OOBS(MP),OFT(MP),
     3 OPR(MP)
      REAL LAMBDA,LAMHAT,BUFFER
      CHARACTER IFMT*55,OFMT*55,ADD*4,TITLE*30
      DATA ADD/'(T6,'/
C
C ZERO ARRAYS.
C
      REWIND 2
      DO 1 L=1,NCOL
      DO 2 M=1,NCOL
2     ZTWZ(L,M)=0.
      ZTWY(L,1)=0.
      PROD1(L)=0.
      PROD2(L)=0.
1     CONTINUE
      BUFFER=0.
      TPR=0.
      DEV=0.
      I=0
      IDF=0
      READ(2,3)TITLE,IFMT
3     FORMAT(A30,/,A50)
      OFMT(4:55)=IFMT
      OFMT(1:4)=ADD
5     CONTINUE
      I=I+1
      WRITE(6,1000)I
1000  FORMAT('+','Records read: ',I6)
      READ(2,IFMT,END=500)OBS,PY,(X(J),J=1,IPRED)
      FT=0.
      PR=0.
      EXPE=0.
      LAMHAT=0.
      W=0.
      Y=0.
      IFAULT=0.
      IF(PY.GT.0.)LAMBDA=OBS*(1./PY)
      IF(NREG.EQ.2)RHO=0.
      IF(NREG.NE.4)THEN
      DO 6 JJ=1,NCOL
      LAMHAT=LAMHAT+X(JJ)*G(JJ)
6     CONTINUE
      ENDIF
      IF(RHO.EQ.0.)LAMHAT=EXP(LAMHAT)
      IF(LAMHAT.GT.0..AND.RHO.NE.0.)LAMHAT=((LAMHAT)**
     1 (1./RHO))
      IF(NREG.EQ.4)THEN
      LAMHAT=G(1)*X(1)*(1. - (1. -EXP(-G(2)*X(2)))**G(3))
      ENDIF
      EXPE=AMAX1(PY*LAMHAT,1.E-12)
      OBS=AMAX1(OBS,1.E-12)
      OOBS(I)=OBS
      Y=OBS-EXPE
      YDIFF(I)=Y
      OEXPE(I)=EXPE
      W=(1./(AMAX1(1.E-9,EXPE)))
      WIN(I,I)=W
      IF(EXPE.GT.0.)FT=SQRT(OBS)+SQRT(OBS+1)-
     1 SQRT(4*EXPE+1)
      PR=((OBS-EXPE)**2.)/(AMAX1(EXPE,1.E-6))
      OFT(I)=FT
      OPR(I)=PR
      IDF=IDF+1
      TPR=TPR+PR
      DEV=DEV+OBS*(ALOG(OBS)-ALOG(1./W))+
     1 (EXPE-OBS)
      IF(NREG.EQ.4)GO TO 8
      DO 7 K=1,NCOL
      Z(K)=0.
      Z(K)=X(K)*PY
      IF(LAMHAT.GT.0..AND.RHO.NE.0.)Z(K)=Z(K)/RHO
     1 *(LAMHAT**(1.-RHO))
      IF(RHO.EQ.0.)Z(K)=Z(K)*LAMHAT
7     CONTINUE
8     CONTINUE
C
C FOR NON-LINEAR MODELS (NREG=4), FILL IN JACOBIAN MATRIX, Z(.) WITH VALUES
C OF THE FIRST PARTIAL DERIVATIVES OF LAMBDA WITH RESPECT TO EACH PARAMETER.
C
      IF(NREG.EQ.4)THEN
      DO 9 K=1,NCOL
9     Z(K)=0.
      Z(1)=PY*G(1)*X(1)*(1.-(1.-EXP(-G(2)*X(2)))**G(3)) 
      IF(X(2).EQ.0.)GO TO 505
      Z(2)=-PY*G(1)*X(1)*G(3)*X(2) * (1.-EXP(-G(2)*X(2)))**G(3) 
     1 * EXP(-G(2)*X(2)) / (1.- EXP(-G(2)*X(2)))
      Z(3)=-PY*G(1)*X(1) * (1.-EXP(-G(2)*X(2)))**G(3) *
     1 ALOG(1.- EXP(-G(2)*X(2)))
      ENDIF
505   CONTINUE
C
C CALCULATE DISPERSION MATRICES.
C
      DO 11 L=1,NCOL
      XT(L,I)=Z(L)
      XIN(I,L)=Z(L)
      DO 12 M=1,NCOL
      ZTWZ(L,M)=ZTWZ(L,M)+Z(L)*W*Z(M)
12    CONTINUE
11    CONTINUE
      DO 13 L=1,NCOL
      ZTWY(L,1)=ZTWY(L,1)+Z(L)*W*Y
13    CONTINUE
      GO TO 5
500   NROW=I-1
 
      CALL MATINV(ZTWZ,ZTWZ1,NCOL)
      CALL MATMUL(ZTWZ1,ZTWY,G,NCOL,NCOL,IP,MP,NP,IP)
      DEV=2*DEV
      IDF=IDF-NCOL
      DO 14 J=1,NCOL
14    BUFFER=BUFFER+ABS(G(J))
      IF(BUFFER.EQ.0.)IFLT=5
      RETURN
      END

      SUBROUTINE MATINV(ZIN,ZOUT,NCOL)
C
C  SUBBROUTINE TO INVERT AN ILL-CONDITIONED POSITIVE REAL SYMMETRIC
C  MATRIX. USES SUBROUTINE SVDCMP FOR SINGULAR VALUE DECOMPOSITION.
C
C  ARRAYS:
C
C  U(J,J)      - UPPER TRIANGULAR FROM SVD
C  UT(J,J)     - TRANSPOSE OF UPPER TRIANGULAR FROM SVD
C  WZM(J,J)    - DIAGONAL WEIGHTS FROM SVD
C  VWZM(J,J)   - CROSS-PRODUCT OF LOWER TRIANGULAR AND DIAGONAL WEIGHTS
C  V(J,J)      - LOWER TRIANGULAR FROM SVD
C  W(I,I)      - INNER PRODUCT MATRIX FROM SVD
C  WZ(J)       - INVERSE VARIANCE WEIGHTING MATRIX
C
      PARAMETER (NP=28)
      DIMENSION ZIN(NP,NP),ZOUT(NP,NP),U(NP,NP),WZ(NP),
     1 UT(NP,NP),WZM(NP,NP),VWZM(NP,NP),V(NP,NP)
      DO 12 J=1,NCOL
      DO 11 K=1,NCOL
      U(J,K)=0.0
      U(J,K)=ZIN(J,K)
11    CONTINUE
12    CONTINUE
      CALL SVDCMP(U,NCOL,NCOL,NP,NP,WZ,V)
      WMAX=0.0
      DO 13 J=1,NCOL
      WZM(J,J)=0.
      IF(WZ(J).GT.WMAX)WMAX=WZ(J)
  13  CONTINUE
      WMIN=WMAX*1.0E-6
      DO 14 J=1,NCOL
      IF(WZ(J).LT.WMIN)WZ(J)=0.
      IF(WZ(J).NE.0.)WZM(J,J)=(1./WZ(J))
14    CONTINUE
C
C  TRANSPOSE U MATRIX, THEN MULTIPLY BY THE CROSS-PRODUCT OF 
C  THE V AND WZ MATRICES. 
C
      CALL TRNPOS(U,UT,NCOL,NCOL,NP,NP)
      CALL MATMUL(V,WZM,VWZM,NCOL,NCOL,NCOL,NP,NP,NP)
      CALL MATMUL(VWZM,UT,ZOUT,NCOL,NCOL,NCOL,NP,NP,NP) 
      RETURN
      END

      SUBROUTINE MATMUL(A,B,C,L,M,N,MP,NP,IP)   
      INTEGER L,M,N
      DIMENSION A(NP,MP),B(MP,IP)
      DIMENSION C(NP,IP)
      INTEGER  I,J,K
      DOUBLE PRECISION D
      DO 120 I=1,L
      DO 110 J=1,N
      D=0.D0
      DO 100 K=1,M
100   D=D+DBLE(A(I,K))*DBLE(B(K,J))
110   C(I,J)=D
120   CONTINUE
      RETURN
      END

      SUBROUTINE TRNPOS(ZIN,OUT,M,N,MP,NP)
      DIMENSION ZIN(MP,NP),OUT(NP,MP)
      DO 100 I=1,M
      DO 100 J=1,N
      OUT(J,I)=0.
100   OUT(J,I)=ZIN(I,J)
      RETURN
      END

      SUBROUTINE SVDCMP(A,M,N,MP,NP,W,V)
C
C  SINGULAR VALUE DECOMPOSITION OF REAL SYMMETRIC MATRIX.
C  SUBROUTINE SVDCMP.  PRESS, W.H., FLANNERY, B.P., TEUKOLSKY, S.A.,
C  VETTERLING, W.T.  IN: NUMERICAL RECIPES.  CAMBRIDGE: CAMBRIDGE 
C  PRESS; 1986.
C
      PARAMETER (NMAX=100)
      DIMENSION A(MP,NP),W(NP),V(NP,NP),RV1(NMAX)
      IF(M.LT.N)PAUSE 'You must augment A with extra zero rows.'
C
C  HOUSEHOLDER REDUCTION TO BIDIAGONAL FORM.
C
      G=0.0
      SCALE=0.0
      ANORM=0.0
      DO 25 I=1,N
      L=I+1
      RV1(I)=SCALE*G
      G=0.0
      S=0.0
      SCALE=0.0
      IF(I.LE.M)THEN
      DO 11 K=I,M
      SCALE=SCALE+ABS(A(K,I))
11    CONTINUE
      IF(SCALE.NE.0.0)THEN
      DO 12 K=I,M
      A(K,I)=A(K,I)*(1./SCALE)
      S=S+A(K,I)*A(K,I)
12    CONTINUE
      F=A(I,I)
      G=-SIGN(SQRT(S),F)
      H=F*G-S
      A(I,I)=F-G
      IF(I.NE.N)THEN
      DO 15 J=L,N
      S=0.0
      DO 13 K=I,M
      S=S+A(K,I)*A(K,J)
13    CONTINUE
      F=S*(1./H)
      DO 14 K=I,M
      A(K,J)=A(K,J)+F*A(K,I)
14    CONTINUE       
15    CONTINUE
      ENDIF
      DO 16 K=I,M
      A(K,I)=SCALE*A(K,I)
16    CONTINUE          
      ENDIF
      ENDIF
      W(I)=SCALE*G
      G=0.0
      S=0.0
      SCALE=0.0
      IF((I.LE.M).AND.(I.NE.N))THEN
      DO 17 K=L,N
      SCALE=SCALE+ABS(A(I,K))
17    CONTINUE
      IF(SCALE.NE.0.0)THEN
      DO 18 K=L,N
      A(I,K)=A(I,K)*(1./SCALE)
      S=S+A(I,K)*A(I,K)
18    CONTINUE
      F=A(I,L)
      G=-SIGN(SQRT(S),F)
      H=F*G-S
      A(I,L)=F-G
      DO 19 K=L,N
      RV1(K)=A(I,K)*(1./H)
19    CONTINUE
      IF(I.NE.M)THEN
      DO 23 J=L,M
      S=0.0
      DO 21 K=L,N
      S=S+A(J,K)*A(I,K)
21    CONTINUE
      DO 22 K=L,N
      A(J,K)=A(J,K)+S*RV1(K)
22    CONTINUE
23    CONTINUE
      ENDIF     
      DO 24 K=L,N
      A(I,K)=SCALE*A(I,K)
24    CONTINUE
      ENDIF
      ENDIF
      ANORM=MAX(ANORM,(ABS(W(I))+ABS(RV1(I))))
25    CONTINUE
C
C  ACCUMULATION OF RIGHT-HAND TRANSFORMATIONS.
C     
      DO 32 I=N,1,-1
      IF(I.LT.N)THEN
      IF(G.NE.0.0)THEN
      DO 26 J=L,N
      V(J,I)=(A(I,J)*(1./A(I,L)))*(1./G)
26    CONTINUE
      DO 29 J=L,N
      S=0.0
      DO 27 K=L,N
      S=S+A(I,K)*V(K,J)
27    CONTINUE
      DO 28 K=L,N
      V(K,J)=V(K,J)+S*V(K,I)
28    CONTINUE
29    CONTINUE
      ENDIF
      DO 31 J=L,N
      V(I,J)=0.0
      V(J,I)= 0.0
31    CONTINUE
      ENDIF
      V(I,I)=1.0
      G=RV1(I)
      L=I
32    CONTINUE
C
C  ACCUMULATION OF LEFT-HAND TRANSFORMATIONS.
C
      DO 39 I=N,1,-1
      L=I+1
      G=W(I)
      IF(I.LT.N)THEN
      DO 33 J=L,N
      A(I,J)=0.0
33    CONTINUE
      ENDIF
      IF(G.NE.0.0)THEN
      G=1.0*(1./G)
      IF(I.NE.N)THEN
      DO 36 J=L,N
      S=0.0
      DO 34 K=L,M
      S=S+A(K,I)*A(K,J)
34    CONTINUE
      F=(S/A(I,I))*G
      DO 35 K=I,M
      A(K,J)=A(K,J)+F*A(K,I)
35    CONTINUE
36    CONTINUE
      ENDIF
      DO 37 J=I,M
      A(J,I)=A(J,I)*G
37    CONTINUE
      ELSE
      DO 38 J=I,M
      A(J,I)=0.0
38    CONTINUE
      ENDIF
      A(I,I)=A(I,I)+1.0
39    CONTINUE
C
C  DIAGONALIZATION OF THE BIDIAGONAL FORM.
C
      DO 49 K=N,1,-1
      DO 48 ITS=1,30
      DO 41 L=K,1,-1
      NM=L-1
      IF((ABS(RV1(L))+ANORM).EQ.ANORM) GO TO 2
      IF((ABS(W(NM))+ANORM).EQ.ANORM) GO TO 1
41    CONTINUE
1     C=0.0
      S=1.0
      DO 43 I=L,K
      F=S*RV1(I)
      IF((ABS(F)+ANORM).NE.ANORM)THEN
      G=W(I)
      H=SQRT(F*F+G*G)
      W(I)=H
      H=1.0*(1./H)
      C=(G*H)
      S=-(F*H)
      DO 42 J=1,M
      Y=A(J,NM)
      Z=A(J,I)
      A(J,NM)=(Y*C)+(Z*S)
      A(J,I)=-(Y*S)+(Z*C)
42    CONTINUE
      ENDIF
43    CONTINUE
2     Z=W(K)
      IF(L.EQ.K)THEN
      IF(Z.LT.0.0)THEN
      W(K)=-Z
      DO 44 J=1,N
      V(J,K)=-V(J,K)
44    CONTINUE
      ENDIF
      GO TO 3
      ENDIF
      IF(ITS.EQ.30) PAUSE 'No convergence in 30 iterations.'
      X=W(L)
      NM=K-1
      Y=W(NM)
      G=RV1(NM)
      H=RV1(K)
      F=((Y-Z)*(Y+Z)+(G-H)*(G+H))/(2.0*H*Y)
      G=SQRT(F*F+1.0)
      F=((X-Z)*(X+Z)+H*((Y/(F+SIGN(G,F)))-H))/X       
C
C  NEXT QR TRANSFORMATION
C
      C=1.0
      S=1.0
      DO 47 J=L,NM
      I=J+1
      G=RV1(I)
      Y=W(I)
      H=S*G
      G=C*G
      Z=SQRT(F*F+H*H)
      RV1(J)=Z
      C=F*(1./Z)
      S=H*(1./Z)
      F=(X*C)+(G*S)
      G=-(X*S)+(G*C)
      H=Y*S
      Y=Y*C
      DO 45 JJ=1,N
      X=V(JJ,J)
      Z=V(JJ,I)
      V(JJ,J)=(X*C)+(Z*S)
      V(JJ,I)=-(X*S)+(Z*C)
45    CONTINUE
      Z=SQRT(F*F+H*H)
      W(J)=Z
      IF(Z.NE.0.0)THEN
      Z=1.0*(1./Z)
      C=F*Z
      S=H*Z
      ENDIF
      F=(C*G)+(S*Y)
      X=-(S*G)+(C*Y)
      DO 46 JJ=1,M
      Y=A(JJ,J)
      Z=A(JJ,I)
      A(JJ,J)=(Y*C)+(Z*S)
      A(JJ,I)=-(Y*S)+(Z*C)
46    CONTINUE
47    CONTINUE
      RV1(L)=0.0
      RV1(K)=F
      W(K)=X
48    CONTINUE
3     CONTINUE
49    CONTINUE
      RETURN
      END


      FUNCTION ALG(GP,IF1)
C
C  A.C.M. ALGORITHM 291:: EVALUATE LOG-GAMMA FUNCTION
C
      X=GP
      IF1 = 2
      IF(X.LE.0.0)RETURN
      IF1 = 0
      F = 0.0
      IF(X.GE.7.0) GO TO 40
      F = 1.0
      Z = X - 1.0
   20 Z = Z + 1.0
      IF( Z.GE.7.0 ) GO TO 30
      X = Z
      F = F * Z
      GO TO 20
   30 X = X + 1.0
      F = -ALOG(F)
   40 Z = 1.0/X**2
   50 ALG = F + (X-0.5)*ALOG(X) - X + .9189385332
      ALG = ALG + 1.0/(12.0*X) - Z/(360.0*X)
      ALG = ALG + Z**2/(1260.0*X) - Z**3/(1680.0*X)
      RETURN
      END

      FUNCTION GAMAIN(X,P,G,IFAULT)
C
C         ALGORITHM AS32 J.R.STATIST.SOC.C.(1970) VOL.19 NO.3
C
      DIMENSION PN(6)
      ACU=1.0E-8
      OFLO=1.0E30
      GIN=0.0
      IFAULT=0
C
      IF(P.LE.0.0) IFAULT=1
      IF(X.LT.0.0) IFAULT=2
      IF( IFAULT.GT.0.OR.X.EQ.0.0 )GO TO 50
      FACTOR=EXP(P*ALOG(X)-X-G)
      IF(X.GT.1.0.AND.X.GE.P) GO TO 30
C
      GIN=1.0
      TERM=1.0
      RN=P
   20 RN=RN+1.0
      TERM=TERM*X/RN
      GIN=GIN+TERM
      IF(TERM.GT.ACU) GO TO 20
      GIN=GIN*FACTOR/P
      GO TO 50
C
   30 A=1.0-P
      B=A+X+1.0
      TERM=0.0
      PN(1) = 1.0
      PN(2) = X
      PN(3) = X+1.0
      PN(4) = X*B
      GIN=PN(3)/PN(4)
   32 A=A+1.0
      B=B+2.0
      TERM=TERM+1.0
      AN=A*TERM
      DO 33 I=1,2
   33 PN(I+4)=B*PN(I+2)-AN*PN(I)
      IF(PN(6).EQ.0.0) GO TO 35
      RN=PN(5)/PN(6)
      DIF=ABS(GIN-RN)
      IF(DIF.GT.ACU)GO TO 34
      IF(DIF.LE.ACU*RN) GO TO 42
   34 GIN=RN
   35 DO 36 I=1,4
   36 PN(I) = PN(I+2)
      IF(ABS(PN(5)).LT.OFLO) GO TO 32
      DO 41 I=1,4
   41 PN(I) = PN(I)/OFLO
      GO TO 32
   42 GIN=1.0-FACTOR*GIN
C
   50 GAMAIN=GIN
      RETURN
      END

      SUBROUTINE FIT(W,Y,Z,ZT,ZTWZ,EXPE,LEV,DRES,SRES,NCOL,NROW)
      PARAMETER (MP=500,NP=28,IP=1,NIP=16)   
      DIMENSION Z(MP,NP),ZT(NP,MP),ZTWZ(NP,NP),W(MP,MP),
     1 DRES(MP,NP),SRES(MP),WSQR(MP,MP),ZTW(NP,MP),WZ(MP,NP),RES(NP),
     2 A(MP,NP),Y(MP),SCALE(NP),EXPE(MP)
      REAL LEV(MP,MP)
      DO 10 J=1,NROW
10    WSQR(J,J)=SQRT(W(J,J))
      CALL MATMUL(ZT,WSQR,ZTW,NCOL,NROW,NROW,MP,NP,MP)
      CALL MATMUL(WSQR,Z,WZ,NROW,NROW,NCOL,MP,MP,NP)
      CALL MATMUL(WZ,ZTWZ,A,NROW,NCOL,NCOL,NP,MP,NP)
      CALL MATMUL(A,ZTW,LEV,NROW,NCOL,NROW,NP,MP,MP)
C
C ESTIMATE DELETION RESIDUALS TO DETERMINE CONSISTENCY.
C
      DO 30 J=1,NROW
      DO 20 K=1,NCOL
      ZTWZ(J,K)=-ZTWZ(J,K)
20    CONTINUE
30    CONTINUE
      IONE=1
      DO 50 J=1,NROW
      DO 40 K=1,NCOL
40    SCALE(K)=(Z(J,K)*Y(J)*W(J,J))*(1./(1.-LEV(J,J)))
      CALL MATMUL(ZTWZ,SCALE,RES,NCOL,NCOL,IONE,NP,NP,IP)
      DO 60 L=1,NCOL
60    DRES(J,L)=RES(L)
50    CONTINUE
C
C ESTIMATE ADJUSTED RESIDUALS.
C 
      DO 70 J=1,NROW
70    SRES(J)=Y(J)*(1./SQRT(EXPE(J)*(1.-LEV(J,J))))
      RETURN
      END

      SUBROUTINE FAULT(IFLT)
      IF(IFLT.EQ.1)THEN
      WRITE(*,*)'  FAULT = 1, NUMBER OF PARAMETERS LESS THAN ONE'
      GO TO 5
      ENDIF
      IF(IFLT.EQ.2)THEN
      WRITE(*,*)'  FAULT = 2, NUMBER OF RECORDS LESS THAN ONE'
      GO TO 5
      ENDIF
      IF(IFLT.EQ.3)THEN
      WRITE(*,*)'  FAULT = 3, DEGREES OF FREEDOM LESS THAN ONE'
      GO TO 5
      ENDIF
      IF(IFLT.EQ.4)THEN
      WRITE(*,*)'  FAULT = 4, NULL PARAMETER VECTOR'
      GO TO 5
      ENDIF
      IF(IFLT.EQ.5)THEN
      WRITE(*,*)'  FAULT = 5, NULL SOLUTION VECTOR'
      GO TO 5
      ENDIF
      IF(IFLT.EQ.6)THEN
      WRITE(*,*)'  FAULT = 6, ITERATIONS GREATER THAN 100'
      ENDIF
5     CONTINUE
      STOP
      END
